local core = LibStub("AceAddon-3.0"):NewAddon("LootSink", "AceConsole-3.0", "AceEvent-3.0", "LibSink-2.0")

local db

function core:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("LootSinkDB", {
		profile = {
			total = true,
			money = true,
			longmoney = false,
			sink = {},
		}
	})
	db = self.db.profile
	self:SetSinkStorage(db.sink)
	
	local function getOptions()
		local options = {
			name = "LootSink",
			type = "group",
			get = function(info) return db[info[#info]] end,
			set = function(info, value) db[info[#info]] = value end,
			args = {
				money = {
					type = "toggle",
					name = "Money", desc = "Announce cash loot",
					order = 10,
				},
				longmoney = {
					type = "toggle",
					name = "Long money", desc = "Use long-form money strings ('Copper' instead of 'c')",
					order = 20,
				},
				total = {
					type = "toggle",
					name = "Total", desc = "Show the new total number of an item that you are carrying",
					order = 30,
				},
			},
		}
		return options
	end
	local function getOutput()
		local output = {
			name = "Message Output",
			type = "group",
			args = {
				desc = {
					type = "description", order = 0,
					name = "You can select where you want LootSink's messages displayed from this screen",
				},
				sink = self:GetSinkAce3OptionsDataTable(),
			},
		}
		output.args.sink.order = 1
		output.args.sink.inline = true
		output.args.sink.name = ""
		return output
	end
	local function getProfiles()
		local profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
		return profiles
	end

	local acreg = LibStub("AceConfigRegistry-3.0")
	local acdia = LibStub("AceConfigDialog-3.0")

	acreg:RegisterOptionsTable("LootSink", getOptions)
	acreg:RegisterOptionsTable("LootSink Output", getOutput)
	acreg:RegisterOptionsTable("LootSink Profiles", getProfiles)

	acdia:AddToBlizOptions("LootSink", "LootSink")
	acdia:AddToBlizOptions("LootSink Profiles", "Profiles", "LootSink")
	acdia:AddToBlizOptions("LootSink Output", "Output", "LootSink")

	local function openblizz() InterfaceOptionsFrame_OpenToFrame(acdia.BlizOptions["LootSink"].frame) end
	self:RegisterChatCommand("lootsink", openblizz)
end

function core:OnEnable()
	self:RegisterEvent("CHAT_MSG_LOOT")
	self:RegisterEvent("CHAT_MSG_MONEY")
end

local loot_pattern = (LOOT_ITEM_SELF):gsub("%%s", "(.+)") -- "You receive loot: %s."
local loot_multiple_pattern = (LOOT_ITEM_SELF_MULTIPLE):gsub("%%sx%%d", "(.+)x(%%d+)") -- "You receive loot: %sx%d."
function core:CHAT_MSG_LOOT()
	--arg1 will be the loot message
	local link, quantity = arg1:match(loot_multiple_pattern)
	if not link then
		link = arg1:match(loot_pattern)
		if not link then
			return
		end
	end
	local name, _, rarity = GetItemInfo(link)
	if name then
		self:Pour("["..name.."]"..
			(quantity and ("x"..quantity) or "")..
			(db.total and (" ("..(GetItemCount(link) + (quantity or 1))..")") or ""),
			GetItemQualityColor(rarity)
		)
	end
end

local gold_pattern = "(%d+) "..GOLD
local silver_pattern = "(%d+) "..SILVER
local copper_pattern = "(%d+) "..COPPER
local money_string_to_copper = function(s)
	return (tonumber(s:match(gold_pattern)) or 0)*10000 + (tonumber(s:match(silver_pattern)) or 0)*100 + (tonumber(s:match(copper_pattern)) or 0)
end
local copper_to_pretty_money = function(c, long)
	if c > 10000 then
		return ("%d|cffffd700%s|r%d|cffc7c7cf%s|r%d|cffeda55f%s|r"):format(c/10000, long and GOLD or 'g', (c/100)%100, long and SILVER or 's', c%100, long and COPPER or 'c')
	elseif c > 100 then
		return ("%d|cffc7c7cf%s|r%d|cffeda55f%s|r"):format((c/100)%100, long and SILVER or 's', c%100, long and COPPER or 'c')
	else
		return ("%d|cffeda55f%s|r"):format(c%100, long and COPPER or 'c')
	end
end

local money_pattern = (YOU_LOOT_MONEY):gsub("%%s", "(.+)") -- "You loot %s"
function core:CHAT_MSG_MONEY()
	if db.money then
		local money = money_string_to_copper(arg1)
		if money and money > 0 then
			self:Pour(copper_to_pretty_money(money))
		end
	end
end

